import Vue from "vue";
import Watcher from "./watcher";
import {arrayFind} from "../utils/util";

Watcher.prototype.mutations = {
  setData(states, data) {
    const dataInstanceChanged = states._data !== data;
    states._data = data;

    this.execQuery();
    // 数据变化，更新部分数据。
    // 没有使用 computed，而是手动更新部分数据 https://github.com/vuejs/vue/issues/6660#issuecomment-331417140
    this.updateCurrentRow();
    this.updateExpandRows();
    if (!states.reserveSelection) {
      if (dataInstanceChanged) {
        this.clearSelection();
      } else {
        this.cleanSelection();
      }
    } else {
      this.assertRowKey();
      this.updateSelectionByRowKey();
    }

    this.updateAllSelected();
    this.updateTableScrollY();

    let row = {};
    if (states.addFlag) {
      row = data[data.length - 1];
      if (states.reverse) {
        row = data[0];
      }
      // 编辑行   status: 显示是否添加，true添加，false移除
      this.toggleRowEdition(null, row, {edit: false, add: true}, true);
    }
  },

  insertColumn(states, column, index, parent) {
    let array = states._columns;
    if (parent) {
      array = parent.children;
      if (!array) {array = parent.children = [];}
    }

    if (typeof index !== "undefined") {
      array.splice(index, 0, column);
    } else {
      array.push(column);
    }

    if (column.type === "selection") {
      states.selectable = column.selectable;
      states.reserveSelection = column.reserveSelection;
    }

    if (this.table.$ready) {
      this.updateColumns(); // hack for dynamics insert column
      this.scheduleLayout();
    }
  },

  removeColumn(states, column, parent) {
    let array = states._columns;
    if (parent) {
      array = parent.children;
      if (!array) {array = parent.children = [];}
    }
    if (array) {
      array.splice(array.indexOf(column), 1);
    }

    if (this.table.$ready) {
      this.updateColumns(); // hack for dynamics remove column
      this.scheduleLayout();
    }
  },

  sort(states, options) {
    const { prop, order } = options;
    if (prop) {
      // TODO：nextTick 是否有必要？
      Vue.nextTick(() => {
        const column = arrayFind(states.columns, (column) => column.property === prop);
        if (column) {
          column.order = order;
          this.updateSort(column, prop, order);
          this.commit("changeSortCondition");
        }
      });
    }
  },

  changeSortCondition(states, options) {
    // 修复 pr https://github.com/ElemeFE/element/pull/15012 导致的 bug
    const { sortingColumn: column, sortProp: prop, sortOrder: order } = states;
    if (order === null) {
      states.sortingColumn = null;
      states.sortProp = null;
    }
    const ingore = { filter: true };
    this.execQuery(ingore);

    if (!options || !options.silent) {
      this.table.$emit("sort-change", {
        column,
        prop,
        order
      });
    }

    this.updateTableScrollY();
  },

  filterChange(states, options) {
    const { column, values, silent } = options;
    const newFilters = this.updateFilters(column, values);

    this.execQuery();

    if (!silent) {
      this.table.$emit("filter-change", newFilters);
    }

    this.updateTableScrollY();
  },

  toggleAllSelection() {
    this.toggleAllSelection();
  },

  rowSelectedChanged(states, row) {
    this.toggleRowSelection(row);
    this.updateAllSelected();
  },

  setHoverRow(states, row) {
    states.hoverRow = row;
  },

  setCurrentRow(states, row) {
    const oldCurrentRow = states.currentRow;
    states.currentRow = row;

    if (oldCurrentRow !== row) {
      this.table.$emit("current-change", row, oldCurrentRow);
    }
  },

  setAddFlag(states, addFlag = false) {
    this.setAddFlag(addFlag);
  },
  setRules(states, rules) {
    this.setRules(rules);
  },
  setAddData(states, data) {
    this.setAddData();
  },
  toggleRowEdition(states, type, row, rowConfig, status) {
    this.toggleRowEdition(type, row, rowConfig, status);
  },
  toggleOperation(states, type, row, rowConfig, status, toggleCallback) {
    this.toggleOperation(type, row, rowConfig, status, toggleCallback);
  }
};

Watcher.prototype.commit = function(name, ...args) {
  const mutations = this.mutations;
  if (mutations[name]) {
    mutations[name].apply(this, [this.states].concat(args));
  } else {
    throw new Error(`Action not found: ${name}`);
  }
};

Watcher.prototype.updateTableScrollY = function() {
  Vue.nextTick(this.table.updateScrollY);
};

export default Watcher;
