import {arrayFindIndex} from "./utils/util";
import {getCell, getColumnByCell, getRowIdentity} from "./utils/util";
import {getStyle, hasClass, removeClass, addClass} from "./utils/dom";
import {debounce} from "throttle-debounce";
import LayoutObserver from "./layout-observer";
import {mapStates} from "./store/helper";
import TableCell from "./table-cell.vue";
import TableCellOperation from "./table-cell-operation.vue";

export default{
  name: "TableRow",
  methods: {

    /*#region 操作函数(handle)*/
    handleContextMenu(event, row) {
      this.handleEvent(event, row, "contextmenu");
    },
    handleClick(event, row) {
      this.store.commit("setCurrentRow", row);
      this.handleEvent(event, row, "click");
    },
    handleDoubleClick(event, row) {
      this.handleEvent(event, row, "dblclick");
    },
    handleMouseEnter: debounce(30, function(index) {
      this.store.commit("setHoverRow", index);
    }),
    handleMouseLeave: debounce(30, function() {
      this.store.commit("setHoverRow", null);
    }),
    handleEvent(event, row, name) {
      const table = this.table;
      const cell = getCell(event);
      let column;
      if (cell) {
        column = getColumnByCell(table, cell);
        if (column) {
          table.$emit(`cell-${name}`, row, column, cell, event);
        }
      }
      table.$emit(`row-${name}`, row, column, event);
    },

    /*#endregion*/


    getRowValidate(callback) {
      const itemStates = [];
      this.$emit("validate", (validateState) => {
        // 验证
        itemStates.push(validateState);
      });
      callback(!itemStates.includes(false));
    },
    getRowResetFields(callback) {
      this.$emit("resetFields", () => {
        callback();
      });
    },
    validate(callback) {
      this.$emit("validate", (validateState, cell) => {
        callback(validateState, cell);
        // console.log("validate", validateState);
      });
    },
    resetFields(callback) {
      this.$emit("resetFields", () => {
        callback();
      });
    },
    isColumnHidden(index) {
      if (this.fixed === true || this.fixed === "left") {
        return index >= this.leftFixedLeafCount;
      } else if (this.fixed === "right") {
        return index < this.columnsCount - this.rightFixedLeafCount;
      } else {
        return (index < this.leftFixedLeafCount) || (index >= this.columnsCount - this.rightFixedLeafCount);
      }
    },
    getCellStyle(rowIndex, columnIndex, row, column) {
      const cellStyle = this.table.cellStyle;
      if (typeof cellStyle === "function") {
        return cellStyle.call(null, {
          rowIndex,
          columnIndex,
          row,
          column
        });
      }
      return cellStyle;
    },
    getCellClass(rowIndex, columnIndex, row, column) {
      const classes = [column.id, column.align, column.className];

      if (this.isColumnHidden(columnIndex)) {
        classes.push("is-hidden");
      }

      const cellClassName = this.table.cellClassName;
      if (typeof cellClassName === "string") {
        classes.push(cellClassName);
      } else if (typeof cellClassName === "function") {
        classes.push(cellClassName.call(null, {
          rowIndex,
          columnIndex,
          row,
          column
        }));
      }

      return classes.join(" ");
    },
    getColspanRealWidth(columns, colspan, index) {
      if (colspan < 1) {
        return columns[index].realWidth;
      }
      const widthArr = columns.map(({ realWidth }) => realWidth).slice(index, index + colspan);
      return widthArr.reduce((acc, width) => acc + width, -1);
    },
    getRowClass(row, rowIndex) {
      const classes = ["xt-table__row"];
      if (this.table.highlightCurrentRow && row === this.store.states.currentRow) {
        classes.push("current-row");
      }

      if (this.stripe && rowIndex % 2 === 1) {
        classes.push("xt-table__row--striped");
      }
      const rowClassName = this.table.rowClassName;
      if (typeof rowClassName === "string") {
        classes.push(rowClassName);
      } else if (typeof rowClassName === "function") {
        classes.push(rowClassName.call(null, {
          row,
          rowIndex
        }));
      }

      if (this.store.states.expandRows.indexOf(row) > -1) {
        classes.push("expanded");
      }

      return classes;
    },
    getRowStyle(row, rowIndex) {
      const rowStyle = this.table.rowStyle;
      if (typeof rowStyle === "function") {
        return rowStyle.call(null, {
          row,
          rowIndex
        });
      }
      return rowStyle || null;
    },
    getSpan(row, column, rowIndex, columnIndex) {
      let rowspan = 1;
      let colspan = 1;
      const fn = this.table.spanMethod;
      if (typeof fn === "function") {
        const result = fn({
          row,
          column,
          rowIndex,
          columnIndex
        });
        if (Array.isArray(result)) {
          rowspan = result[0];
          colspan = result[1];
        } else if (typeof result === "object") {
          rowspan = result.rowspan;
          colspan = result.colspan;
        }
      }
      return { rowspan, colspan };
    },
    getKeyOfRow(row, index) {
      const rowKey = this.table.rowKey;
      if (rowKey) {
        return getRowIdentity(row, rowKey);
      }
      return index;
    }
  },
  render(h) {
    const row = this.row, $index = this.$index, treeRowData = this.treeRowData;
    const { treeIndent, columns, firstDefaultColumnIndex } = this.bodyThis;
    const columnsHidden = columns.map((column, index) => this.isColumnHidden(index));
    const rowClasses = this.getRowClass(row, $index);
    let display = true;
    if (treeRowData) {
      rowClasses.push("xt-table__row--level-" + treeRowData.level);
      display = treeRowData.display;
    }
    const rowKey = this.getKeyOfRow(row, $index);
    return (
      <tr
        v-show={display}
        style={ this.getRowStyle(row, $index) }
        class={ rowClasses } >
      {
        columns.map((column, cellIndex) => {
          const { rowspan, colspan } = this.getSpan(row, column, $index, cellIndex);
          if (!rowspan || !colspan) {
            return null;
          }
          const columnData = { ...column };
          columnData.realWidth = this.getColspanRealWidth(columns, colspan, cellIndex);
          const editFlag = this.store.isEdit(row), addFlag = this.store.isAdd(row);
          const data = {
            store: this.store,
            _self: this.context || this.table.$vnode.context,
            column: columnData,
            row,
            $index,
            edit: editFlag || addFlag,
            rowConfig: {
              edit: editFlag,
              add: addFlag,
              isSelected: this.isSelected,
              _id: Math.random(),
              rowKey,
              rule: this.store.states.rules[column.prop || column.property] || null
            }
          };
          if (cellIndex === firstDefaultColumnIndex && treeRowData) {
            data.treeNode = {
              indent: treeRowData.level * treeIndent,
              level: treeRowData.level
            };
            // TODO: 优化这里的逻辑
            if (typeof treeRowData.expanded === "boolean") {
              data.treeNode.expanded = treeRowData.expanded;
              // 表明是懒加载
              if ("loading" in treeRowData) {
                data.treeNode.loading = treeRowData.loading;
              }
              if ("noLazyChildren" in treeRowData) {
                data.treeNode.noLazyChildren = treeRowData.noLazyChildren;
              }
            }
          }
          // key={data.rowConfig._id}
          return (
            <td
              style={ this.getCellStyle($index, cellIndex, row, column) }
              class={ this.getCellClass($index, cellIndex, row, column) }
              rowspan={ rowspan }
              colspan={ colspan }>
            {
              column.renderCell.call(
                this._renderProxy,
                this.$createElement,
                data,
                columnsHidden[cellIndex]
              )
            }
            </td>
        );
        })
      }
      </tr>
  );
  },
  components: {
    TableCell,
    TableCellOperation
  },
  computed: {
    ...mapStates({
      data: "data",
      columns: "columns",
      treeIndent: "indent",
      leftFixedLeafCount: "fixedLeafColumnsLength",
      rightFixedLeafCount: "rightFixedLeafColumnsLength",
      columnsCount: (states) => states.columns.length,
      leftFixedCount: (states) => states.fixedColumns.length,
      rightFixedCount: (states) => states.rightFixedColumns.length,
      hasExpandColumn: (states) => states.columns.some(({ type }) => type === "expand")
    }),

    firstDefaultColumnIndex() {
      return arrayFindIndex(this.columns, ({ type }) => type === "default");
    },
    isSelected() {
      return this.store.isSelected(this.row);
    }
  },
  data() {
    return {

    };
  },
  props: {
    bodyThis: Object,
    row: Object,
    $index: Number,
    treeRowData: Object,
    store: {
      required: true
    },
    table: {
      required: true
    }
  }

};
